@@ -2,44 +2,146 @@
 Copyright © 2025 John Liu
 """
 
+import importlib.metadata
 import itertools
 import json
 import subprocess
 import tomllib
+from base64 import b64encode
 from datetime import datetime
-from importlib.metadata import version
-from multiprocessing import Pool, cpu_count
+from multiprocessing import Pool, cpu_count, current_process
 from os.path import getmtime, getsize
 from pathlib import Path
+from time import time
 
+import httpx
 import piexif
 import pillow_heif
-from loguru import logger
+from loguru import logger as log
+from packaging import version  # compare versions safely
 from PIL import Image, ImageChops
 from PIL.TiffImagePlugin import IFDRational
 from tqdm import tqdm
 
-from batch_img.const import PATTERNS, PKG_NAME, REPLACE, TS_FORMAT, VER
+from batch_img.const import (
+    EXIF,
+    EXPIRE_HOUR,
+    PATTERNS,
+    PKG_NAME,
+    REPLACE,
+    TS_FORMAT,
+    UNKNOWN,
+    VER,
+)
+from batch_img.log import Log
 
-pillow_heif.register_heif_opener()  # allow Pillow to open HEIC files
+pillow_heif.register_heif_opener()
+VER_CACHE = Path(f"~/.{PKG_NAME}_version_cache.json").expanduser()
 
 
 class Common:
     @staticmethod
-    def get_version() -> str:
-        """
-        Get this package version using several ways
+    def get_version(pkg_name: str) -> str:
+        """Get this package version by various ways
+
+        Args:
+            pkg_name: package name str
+
+        Returns:
+            str:
         """
         try:
-            return version(PKG_NAME)
+            return importlib.metadata.version(pkg_name)
         except (FileNotFoundError, ImportError, ValueError) as e:
-            # Use lazy % formatting in logging for efficiency
-            logger.warning(f"importlib.metadata.version Error: {e}")
-            logger.debug("Try to get version from pyproject.toml file")
+            log.warning(f"importlib.metadata.version() Error: {e}")
+            log.debug("Get version from pyproject.toml file")
             pyproject = Path(__file__).parent.parent / "pyproject.toml"
             with open(pyproject, "rb") as f:
                 return tomllib.load(f)["project"][VER]
 
+    @staticmethod
+    def get_latest_pypi_ver(pkg_name: str, expire_hr: int = EXPIRE_HOUR):
+        """Get the package latest version on PyPI with local cache
+
+        Args:
+            pkg_name: package name str
+            expire_hr: cache expiration hour int
+
+        Returns:
+            str: latest version on PyPI
+        """
+        jsn_url = f"https://pypi.org/pypi/{pkg_name}/json"
+        latest_ver = ""
+        try:
+            if pkg_name in str(VER_CACHE) and VER_CACHE.exists():
+                with open(VER_CACHE, encoding="utf-8") as f:
+                    cache = json.load(f)
+                    if time() - cache["timestamp"] < expire_hr * 3600:
+                        latest_ver = cache["version"]
+            if not latest_ver:
+                response = httpx.get(jsn_url, timeout=5)
+                if response.status_code != 200:
+                    msg = f"⚠️ Error get data from PyPI: {jsn_url}"
+                    log.error(msg)
+                    return UNKNOWN
+                latest_ver = response.json()["info"]["version"]
+                d_cache = {"timestamp": int(time()), "version": latest_ver}
+                with open(VER_CACHE, "w", encoding="utf-8") as f:
+                    json.dump(d_cache, f)
+            return latest_ver
+        except (httpx.RequestError, KeyError, json.JSONDecodeError) as e:
+            raise e
+
+    @staticmethod
+    def check_latest_version(pkg_name: str) -> str:
+        """Check if the installed version is the latest one
+
+        Args:
+            pkg_name: package name str
+
+        Returns:
+            str
+        """
+        msg = ""
+        try:
+            latest_ver = Common.get_latest_pypi_ver(pkg_name)
+            cur_ver = Common.get_version(pkg_name)
+            if version.parse(cur_ver) < version.parse(latest_ver):
+                msg = (
+                    f"🔔 Update available: {cur_ver}  →  {latest_ver}\n"
+                    f"Run '{pkg_name} --update'"
+                )
+                log.info(msg)
+        except (httpx.RequestError, KeyError, json.JSONDecodeError) as e:
+            msg = f"Error get PyPI data: {e}"
+            log.error(msg)
+        return msg
+
+    @staticmethod
+    def update_package(pkg_name: str) -> str:
+        """Update the package to the latest version
+
+        Args:
+            pkg_name: package name str
+
+        Returns:
+            str
+        """
+        Log.init_log_file()
+        msg = Common.check_latest_version(pkg_name)
+        if "Update available" not in msg:
+            return msg
+        log.info(f"🔄 Updating {pkg_name} ...")
+        cmd = f"uv pip install --upgrade {pkg_name}"
+        try:
+            Common.run_cmd(cmd)
+            msg = "✅ Update completed."
+            log.info(msg)
+        except subprocess.CalledProcessError as e:
+            msg = f"❌ Update {pkg_name}: {e}"
+            log.error(msg)
+        return msg
+
     @staticmethod
     def run_cmd(cmd: str) -> tuple:
         """Run a command on the host and get the output
@@ -50,7 +152,7 @@ class Common:
         Returns:
             tuple: returnCode, StdOut, StdErr
         """
-        logger.debug(f"{cmd=}")
+        log.debug(f"{cmd=}")
         try:
             p = subprocess.run(
                 cmd, capture_output=True, text=True, shell=True, check=True
@@ -58,12 +160,25 @@ class Common:
             r_code = p.returncode
             stdout = p.stdout
             stderr = p.stderr
-            logger.debug(f"'{cmd}'\n {r_code=}\n {stdout=}\n {stderr=}")
+            log.debug(f"'{cmd}'\n {r_code=}\n {stdout=}\n {stderr=}")
             return r_code, stdout, stderr
         except subprocess.CalledProcessError as e:
-            logger.exception(e)
+            log.exception(e)
             raise e
 
+    @staticmethod
+    def file_to_base64(file: Path) -> str:
+        """Encode a file to base64 str
+
+        Args:
+            file: input file path
+
+        Returns:
+            str:
+        """
+        with open(file, "rb") as f:
+            return b64encode(f.read()).decode("utf-8")
+
     @staticmethod
     def readable_file_size(in_bytes: int) -> str:
         """Convert bytes to human-readable KB, MB, or GB
@@ -82,11 +197,30 @@ class Common:
         return res
 
     @staticmethod
-    def decode_exif(exif_data: str) -> dict:
+    def remove_exif_gps(exif_data: bytes) -> tuple:
+        """Remove GPS info from the EXIF data
+
+        Args:
+            exif_data: bytes
+
+        Returns:
+            tuple: bool, bytes
+        """
+        exif_dict = piexif.load(exif_data)
+        if "GPS" in exif_dict and exif_dict["GPS"]:
+            exif_dict.pop("GPS")
+            exif_bytes = piexif.dump(exif_dict)
+            log.debug("Removed GPS info in EXIF")
+            return True, exif_bytes
+        log.debug("No GPS in EXIF")
+        return False, exif_data
+
+    @staticmethod
+    def decode_exif(exif_data: bytes) -> dict:
         """Decode the EXIF data
 
         Args:
-            exif_data: str
+            exif_data: bytes
 
         Returns:
             dict
@@ -101,7 +235,7 @@ class Common:
             for tag_id, value in val.items():
                 tag_name = piexif.TAGS[ifd_name].get(tag_id, {}).get("name", tag_id)
                 _dict[tag_name] = value
-        # logger.info(f"{_dict=}")
+        # log.info(f"{_dict=}")
         for key in (
             "FNumber",
             "FocalLength",
@@ -117,7 +251,6 @@ class Common:
         keys = list(_dict.keys())
         for keyword in (
             "DateTime",
-            "GPS",
             "OffsetTime",
             "SubSecTime",
             "Tile",
@@ -132,7 +265,7 @@ class Common:
         _res = {
             k: (v.decode() if isinstance(v, bytes) else v) for k, v in _dict.items()
         }
-        logger.info(f"{_res=}")
+        log.debug(f"{_res=}")
         return _res
 
     @staticmethod
@@ -160,9 +293,9 @@ class Common:
             for key in ("icc_profile", "xmp"):
                 if key in img.info:
                     img.info.pop(key)
-            if "exif" in img.info:
-                exif_data = img.info.pop("exif")
-                d_info["exif"] = Common.decode_exif(exif_data)
+            if EXIF in img.info:
+                exif_data = img.info.pop(EXIF)
+                d_info[EXIF] = Common.decode_exif(exif_data)
 
         return data, d_info
 
@@ -191,13 +324,13 @@ class Common:
         data1, meta1 = Common.get_image_data(path1)
         data2, meta2 = Common.get_image_data(path2)
 
-        logger.info(
-            f"{path1}:\n{json.dumps(meta1, indent=2, default=Common.jsn_serial)}"
-        )
-        logger.info(
-            f"{path2}:\n{json.dumps(meta2, indent=2, default=Common.jsn_serial)}"
-        )
-        return ImageChops.difference(data1, data2).getbbox() is None
+        s1 = f"{path1}:\n{json.dumps(meta1, indent=2, default=Common.jsn_serial)}"
+        log.debug(s1)
+        s2 = f"{path2}:\n{json.dumps(meta2, indent=2, default=Common.jsn_serial)}"
+        log.debug(s2)
+        is_equal = ImageChops.difference(data1, data2).getbbox() is None
+        log.debug(f"{is_equal=}")
+        return is_equal
 
     @staticmethod
     def get_crop_box(width, height, border_width) -> tuple[float, float, float, float]:
@@ -217,6 +350,27 @@ class Common:
         crop_bottom = height - border_width
         return crop_left, crop_top, crop_right, crop_bottom
 
+    @staticmethod
+    def calculate_new_size(width: int, height: int, max_len: int) -> tuple[int, int]:
+        """Calculate the new size with the same aspect ratio
+
+        Args:
+            width: image width int
+            height: image height int
+            max_len: max length int
+
+        Returns:
+            tuple: new_width, new_height
+        """
+        # Calculate to keep aspect ratio
+        if width > height:
+            ratio = max_len / width
+        else:
+            ratio = max_len / height
+        new_width = int(width * ratio)
+        new_height = int(height * ratio)
+        return new_width, new_height
+
     @staticmethod
     def prepare_all_files(in_path: Path, out_path: Path | str):
         """
@@ -230,37 +384,39 @@ class Common:
         """
         if out_path != REPLACE:
             out_path.mkdir(parents=True, exist_ok=True)
-        _files = itertools.chain.from_iterable(in_path.glob(p) for p in PATTERNS)
+        # Fix Path.glob() got 2x count on Windows 10
+        tmp = [in_path.glob(p, case_sensitive=True) for p in PATTERNS]
+        _files = itertools.chain.from_iterable(set(tmp))
         return _files
 
     @staticmethod
-    def multiprocess_progress_bar(func, desc, tasks: list) -> int:
+    def multiprocess_progress_bar(func, desc: str, all_cnt: int, tasks: list) -> int:
         """Run task in multiprocess with progress bar
 
         Args:
             func: function to be run in multiprocess
             desc: description str
+            all_cnt: all tasks cnt int
             tasks: tasks list for multiprocess pool
 
         Returns:
             int: success_cnt
         """
         success_cnt = 0
-        files_cnt = len(tasks)
         workers = max(cpu_count(), 4)
 
         with Pool(workers) as pool:
-            with tqdm(total=files_cnt, desc=desc) as pbar:
-                for ok, res in pool.starmap(func, tasks):
+            with tqdm(total=all_cnt, desc=desc) as pbar:
+                for ok, res in pool.imap_unordered(func, tasks):
                     if ok:
                         success_cnt += 1
                     else:
                         tqdm.write(f"Error: {res}")
-                    pbar.update()
+                    pbar.update(1)
         return success_cnt
 
     @staticmethod
-    def set_out_file(in_path: Path, out_path: Path, extra: str) -> Path:
+    def set_out_file(in_path: Path, out_path: Path, extra: str = "") -> Path:
         """Set the output file path
 
         Args:
@@ -271,9 +427,23 @@ class Common:
         Returns:
             Path:
         """
+        if out_path == REPLACE:
+            out_file = Path(f"{in_path.parent}/{in_path.stem}_tmp{in_path.suffix}")
+            return out_file
         out_path.mkdir(parents=True, exist_ok=True)
         out_file = out_path
         if out_path.is_dir():
             filename = f"{in_path.stem}_{extra}{in_path.suffix}"
             out_file = Path(f"{out_path}/{filename}")
         return out_file
+
+    @staticmethod
+    def set_log_by_process() -> None:
+        """Do log setting in a worker process.
+        loguru config doesn’t propagate across processes.
+
+        Returns:
+            None
+        """
+        if current_process().name != "MainProcess":
+            Log.set_worker_log()
