@@ -4,11 +4,11 @@ Copyright © 2025 John Liu
 
 PKG_NAME = "batch_img"
 VER = "version"
-NAME = "name"
+EXPIRE_HOUR = 48
 UNKNOWN = "unknown"
 
 MSG_OK = "✅ Processed the image file(s)"
-MSG_BAD = "❌ Failed to process the image file(s)."
+MSG_BAD = "❌ Failed to process image file(s)."
 
 TS_FORMAT = "%Y-%m-%d_%H-%M-%S"
 PATTERNS = (
@@ -21,7 +21,11 @@ PATTERNS = (
     "*.PNG",
     "*.png",
 )
-MAX_LENGTH = 1280
-BD_WIDTH = 5
-BD_COLOR = "green"
 REPLACE = "replace"
+EXIF = "exif"
+# Resize to 1920-pixel max length
+# Add 5-pixel width black color border
+# Remove GPS location info
+MAX_LENGTH = 1920
+BD_WIDTH = 5
+BD_COLOR = "black"
